@@ -4,7 +4,6 @@ use std::net::{IpAddr, Ipv4Addr, SocketAddr, SocketAddrV4};
 use std::process::{Command, Stdio};
 use std::sync::mpsc::{channel, Receiver, Sender};
 use std::time;
-use std::time::SystemTime;
 
 use clap::{App, Arg};
 use log::*;
@@ -12,25 +11,13 @@ use yaml_rust::YamlLoader;
 
 use wg_netmanager::configuration::*;
 use wg_netmanager::crypt_udp::CryptUdp;
+use wg_netmanager::crypt_udp::UdpPacket;
 use wg_netmanager::error::*;
+use wg_netmanager::event::Event;
 use wg_netmanager::manager::*;
+use wg_netmanager::tui_display::TuiApp;
 use wg_netmanager::wg_dev::*;
 
-#[derive(Debug)]
-enum Event {
-    Udp(UdpPacket, SocketAddr),
-    PeerListChange,
-    CtrlC,
-    SendAdvertisement { to: SocketAddr },
-    SendAdvertisementToPublicPeers,
-    SendPingToAllDynamicPeers,
-    SendRouteDatabaseRequest { to: SocketAddrV4 },
-    SendRouteDatabase { to: SocketAddrV4 },
-    CheckAndRemoveDeadDynamicPeers,
-    UpdateRoutes,
-    TimerTick1s,
-}
-
 // ===================== Logging Set Up =====================
 fn set_up_logging(log_filter: log::LevelFilter) {
     use fern::colors::*;
@@ -92,6 +79,35 @@ fn main() -> BoxResult<()> {
                 .help("Custom config file in yaml-style")
                 .takes_value(true),
         )
+        .arg(
+            Arg::with_name("existing_interface")
+                .short("e")
+                .long("existing_wg")
+                .help("Use an existing wireguard interface and do not try to create one"),
+        )
+        .arg(
+            Arg::with_name("wireguard_port")
+                .short("w")
+                .long("wireguard-port")
+                .value_name("PORT")
+                .default_value("55555")
+                .help("Wireguard udp port aka Listen port, if not defined in config file")
+                .takes_value(true),
+        )
+        .arg(
+            Arg::with_name("admin_port")
+                .short("u")
+                .long("admin-port")
+                .value_name("PORT")
+                .default_value("50000")
+                .help("udp port for encrypted communication, if not defined in config file")
+                .takes_value(true),
+        )
+        .arg(
+            Arg::with_name("tui")
+                .short("t")
+                .help("Use text user interface"),
+        )
         .arg(
             Arg::with_name("v")
                 .short("v")
@@ -125,11 +141,28 @@ fn main() -> BoxResult<()> {
         3 => log::LevelFilter::Debug,
         _ => log::LevelFilter::Trace,
     };
-    set_up_logging(log_filter);
+
+    let use_tui = matches.is_present("tui");
+    let use_existing_interface = matches.is_present("existing_interface");
+
+    // Select logger based on command line flag
+    //
+    if use_tui {
+        tui_logger::init_logger(log::LevelFilter::Trace).unwrap();
+        tui_logger::set_default_level(log::LevelFilter::Trace);
+    } else {
+        set_up_logging(log_filter);
+    }
 
     let interface = matches.value_of("interface").unwrap();
     let wg_ip: Ipv4Addr = matches.value_of("wg_ip").unwrap().parse().unwrap();
+    let wg_port: u16 = matches.value_of("wireguard_port").unwrap().parse().unwrap();
+    let admin_port: u16 = matches.value_of("admin_port").unwrap().parse().unwrap();
     let computer_name = matches.value_of("name").unwrap();
+    #[cfg(target_os = "linux")]
+    let ip_list = wg_netmanager::interfaces::get();
+    #[cfg(not(target_os = "linux"))]
+    let ip_list = vec![];
 
     let config = matches.value_of("config").unwrap_or("network.yaml");
 
@@ -146,14 +179,14 @@ fn main() -> BoxResult<()> {
 
     let mut peers: Vec<PublicPeer> = vec![];
     for p in conf[0]["peers"].as_vec().unwrap() {
-        info!("PEER: {:#?}", p);
+        info!("STATIC PEER: {:#?}", p);
         let public_ip: IpAddr = p["publicIp"].as_str().unwrap().parse().unwrap();
-        let comm_port = p["wgPort"].as_i64().unwrap() as u16;
+        let wg_port = p["wgPort"].as_i64().unwrap() as u16;
         let admin_port = p["adminPort"].as_i64().unwrap() as u16;
         let wg_ip: Ipv4Addr = p["wgIp"].as_str().unwrap().parse().unwrap();
         let pp = PublicPeer {
             public_ip,
-            comm_port,
+            wg_port,
             admin_port,
             wg_ip,
         };
@@ -178,10 +211,7 @@ fn main() -> BoxResult<()> {
     let mut public_key = String::new();
     cmd.stdout.unwrap().read_to_string(&mut public_key)?;
     let my_public_key = public_key.trim();
-    let timestamp = SystemTime::now()
-        .duration_since(SystemTime::UNIX_EPOCH)
-        .unwrap()
-        .as_secs();
+    let timestamp = wg_netmanager::util::now();
     let my_public_key_with_time = PublicKeyWithTime {
         key: my_public_key.to_string(),
         priv_key_creation_time: timestamp,
@@ -190,17 +220,23 @@ fn main() -> BoxResult<()> {
 
     let static_config = StaticConfiguration::builder()
         .name(computer_name)
+        .ip_list(ip_list)
         .wg_ip(wg_ip)
         .wg_name(interface)
+        .wg_port(wg_port)
+        .admin_port(admin_port)
         .shared_key(shared_key)
         .my_public_key(my_public_key_with_time)
         .my_private_key(my_private_key)
         .peers(peers)
+        .use_tui(use_tui)
+        .use_existing_interface(use_existing_interface)
         .build();
-    run(static_config)
+
+    run(&static_config)
 }
 
-fn run(static_config: StaticConfiguration) -> BoxResult<()> {
+fn run(static_config: &StaticConfiguration) -> BoxResult<()> {
     let (tx, rx) = channel();
 
     let tx_handler = tx.clone();
@@ -212,7 +248,7 @@ fn run(static_config: StaticConfiguration) -> BoxResult<()> {
     })
     .expect("Error setting Ctrl-C handler");
 
-    let port = static_config.my_admin_port().unwrap_or(0);
+    let port = static_config.my_admin_port();
     debug!("bind to 0.0.0.0:{}", port);
     let crypt_socket = CryptUdp::bind(port)?.key(&static_config.shared_key)?;
 
@@ -252,27 +288,43 @@ fn run(static_config: StaticConfiguration) -> BoxResult<()> {
     });
 
     let wg_dev = WireguardDeviceLinux::init(&static_config.wg_name);
-    // in case there are dangling devices
-    wg_dev.take_down_device().ok();
 
-    wg_dev.bring_up_device()?;
+    // in case there are dangling routes
+    if !static_config.use_existing_interface {
+        wg_dev.take_down_device().ok();
+
+        wg_dev.bring_up_device()?;
+    } else {
+        wg_dev.flush_all()?;
+    }
+
     wg_dev.set_ip(&static_config.wg_ip)?;
 
-    let rc = main_loop(static_config, &wg_dev, crypt_socket, tx, rx);
+    let mut tui_app = if static_config.use_tui {
+        TuiApp::init(tx.clone())?
+    } else {
+        TuiApp::off()
+    };
 
-    wg_dev.take_down_device().ok();
+    let rc = main_loop(static_config, &wg_dev, crypt_socket, tx, rx, &mut tui_app);
+
+    if !static_config.use_existing_interface {
+        wg_dev.take_down_device().ok();
+    }
+
+    tui_app.deinit()?;
 
     rc
 }
 
 fn main_loop(
-    static_config: StaticConfiguration,
+    static_config: &StaticConfiguration,
     wg_dev: &dyn WireguardDevice,
     crypt_socket: CryptUdp,
     tx: Sender<Event>,
     rx: Receiver<Event>,
+    tui_app: &mut TuiApp,
 ) -> BoxResult<()> {
-    let mut dynamic_peers = DynamicPeerList::default();
     let mut network_manager = NetworkManager::new(static_config.wg_ip);
 
     // set up initial wireguard configuration without peers
@@ -281,7 +333,7 @@ fn main_loop(
 
     let mut tick_cnt = 0;
     loop {
-        trace!("Main loop: {} peers", dynamic_peers.peer.len());
+        trace!("Main loop: {} peers", network_manager.peer.len());
         let evt = rx.recv();
         trace!("{:?}", evt);
         match evt {
@@ -293,6 +345,8 @@ fn main_loop(
                 break;
             }
             Ok(Event::TimerTick1s) => {
+                tui_app.draw()?;
+
                 if tick_cnt % 15 == 1 {
                     // every 15s
                     tx.send(Event::CheckAndRemoveDeadDynamicPeers).unwrap();
@@ -303,7 +357,7 @@ fn main_loop(
                 }
                 if tick_cnt % 30 == 2 {
                     // every 30s
-                    info!("Main loop: {} peers", dynamic_peers.peer.len());
+                    info!("Main loop: {} peers", network_manager.peer.len());
                 }
                 if tick_cnt % 60 == 3 {
                     // every 60s
@@ -314,7 +368,7 @@ fn main_loop(
             Ok(Event::SendPingToAllDynamicPeers) => {
                 // Pings are sent out only via the wireguard interface.
                 //
-                let ping_peers = dynamic_peers.check_ping_timeouts(10); // should be < half of dead peer timeout
+                let ping_peers = network_manager.check_ping_timeouts(10); // should be < half of dead peer timeout
                 for (wg_ip, admin_port) in ping_peers {
                     let destination = SocketAddr::V4(SocketAddrV4::new(wg_ip, admin_port));
                     tx.send(Event::SendAdvertisement { to: destination })
@@ -326,7 +380,7 @@ fn main_loop(
                 // As all udp packets are encrypted, this should not be an issue.
                 //
                 for peer in static_config.peers.iter() {
-                    if !dynamic_peers.knows_peer(&peer.wg_ip) {
+                    if !network_manager.knows_peer(&peer.wg_ip) {
                         // ensure not to send to myself
                         if peer.wg_ip != static_config.wg_ip {
                             let destination = SocketAddr::new(peer.public_ip, peer.admin_port);
@@ -339,9 +393,9 @@ fn main_loop(
             Ok(Event::Udp(udp_packet, src_addr)) => {
                 use UdpPacket::*;
                 match udp_packet {
-                    Advertisement { .. } => {
+                    Advertisement(ad) => {
                         if let Some(new_wg_ip) =
-                            dynamic_peers.analyze_advertisement(&udp_packet, src_addr.port())
+                            network_manager.analyze_advertisement_for_new_peer(&ad, src_addr.port())
                         {
                             info!("Unknown peer {}", src_addr);
                             network_manager.add_dynamic_peer(&new_wg_ip);
@@ -357,14 +411,14 @@ fn main_loop(
                         } else {
                             info!("Existing peer {}", src_addr);
                         }
-                        if let Some(wg_ip) = network_manager.analyze_advertisement(&udp_packet) {
+                        if let Some(wg_ip) = network_manager.analyze_advertisement(&ad) {
                             // need to request new route database
                             let destination = SocketAddrV4::new(wg_ip, src_addr.port());
                             tx.send(Event::SendRouteDatabaseRequest { to: destination })
                                 .unwrap();
                         }
                     }
-                    RouteDatabaseRequest { .. } => {
+                    RouteDatabaseRequest => {
                         info!("RouteDatabaseRequest from {:?}", src_addr);
                         match src_addr {
                             SocketAddr::V4(destination) => {
@@ -376,9 +430,9 @@ fn main_loop(
                             }
                         }
                     }
-                    RouteDatabase { sender, .. } => {
-                        info!("RouteDatabase for {}", sender);
-                        if network_manager.process_route_database(udp_packet) {
+                    RouteDatabase(req) => {
+                        info!("RouteDatabase from {}", src_addr);
+                        if network_manager.process_route_database(req) {
                             tx.send(Event::UpdateRoutes).unwrap();
                         }
                     }
@@ -387,7 +441,7 @@ fn main_loop(
             Ok(Event::SendAdvertisement { to: destination }) => {
                 let routedb_version = network_manager.db_version();
                 let advertisement =
-                    UdpPacket::advertisement_from_config(&static_config, routedb_version);
+                    UdpPacket::advertisement_from_config(static_config, routedb_version);
                 let buf = serde_json::to_vec(&advertisement).unwrap();
                 info!("Send advertisement to {}", destination);
                 crypt_socket.send_to(&buf, destination).ok();
@@ -407,12 +461,11 @@ fn main_loop(
                 }
             }
             Ok(Event::CheckAndRemoveDeadDynamicPeers) => {
-                dynamic_peers.output();
-                let dead_peers = dynamic_peers.check_timeouts(120);
+                network_manager.output();
+                let dead_peers = network_manager.check_timeouts(120);
                 if !dead_peers.is_empty() {
                     for wg_ip in dead_peers {
                         info!("Found dead peer {}", wg_ip);
-                        dynamic_peers.remove_peer(&wg_ip);
                         network_manager.remove_dynamic_peer(&wg_ip);
                     }
                     tx.send(Event::PeerListChange).unwrap();
@@ -421,7 +474,7 @@ fn main_loop(
             }
             Ok(Event::PeerListChange) => {
                 info!("Update peers");
-                let conf = static_config.as_conf_as_peer(Some(&dynamic_peers), &network_manager);
+                let conf = static_config.as_conf_as_peer(&network_manager);
                 info!("Configuration as peer\n{}\n", conf);
                 wg_dev.sync_conf(&conf)?;
             }
@@ -447,6 +500,10 @@ fn main_loop(
                 }
                 tx.send(Event::PeerListChange).unwrap();
             }
+            Ok(Event::TuiApp(evt)) => {
+                tui_app.process_event(evt);
+                tui_app.draw()?;
+            }
         }
     }
     Ok(())
