@@ -24,13 +24,13 @@
 use std::collections::hash_map::Entry;
 use std::collections::HashMap;
 use std::collections::HashSet;
-use std::net::Ipv4Addr;
-//use std::net::SocketAddr;
+use std::net::{IpAddr, Ipv4Addr, SocketAddr};
 
 use log::*;
 use serde::{Deserialize, Serialize};
 
 use crate::configuration::*;
+use crate::crypt_udp::*;
 
 #[derive(Debug)]
 pub enum RouteChange {
@@ -76,12 +76,29 @@ pub struct PeerRouteDB {
     route_for: HashMap<Ipv4Addr, RouteInfo>,
 }
 
+#[derive(Debug)]
+pub struct DynamicPeer {
+    pub public_key: PublicKeyWithTime,
+    pub local_ip_list: Vec<IpAddr>,
+    pub local_wg_port: u16,
+    pub local_admin_port: u16,
+    pub wg_ip: Ipv4Addr,
+    pub name: String,
+    pub endpoint: Option<SocketAddr>,
+    pub admin_port: u16,
+    pub lastseen: u64,
+}
+
 pub struct NetworkManager {
     wg_ip: Ipv4Addr,
     //all_nodes: HashMap<Ipv4Addr, NodeInfo>,
     peers: HashSet<Ipv4Addr>,
     route_db: RouteDB,
     peer_route_db: HashMap<Ipv4Addr, PeerRouteDB>,
+
+    pub peer: HashMap<Ipv4Addr, DynamicPeer>,
+    pub fifo_dead: Vec<Ipv4Addr>,
+    pub fifo_ping: Vec<Ipv4Addr>,
 }
 
 impl NetworkManager {
@@ -92,33 +109,21 @@ impl NetworkManager {
             peers: HashSet::new(),
             route_db: RouteDB::default(),
             peer_route_db: HashMap::new(),
+
+            peer: HashMap::new(),
+            fifo_dead: vec![],
+            fifo_ping: vec![],
         }
     }
 
     pub fn add_dynamic_peer(&mut self, peer_ip: &Ipv4Addr) {
         self.peers.insert(*peer_ip);
         self.recalculate_routes();
-
-        // Dynamic peers are ALWAYS reachable without a gateway
-        let ri = RouteInfoWithStatus {
-            ri: RouteInfo {
-                to: *peer_ip,
-                gateway: None,
-            },
-            issued: false,
-            to_be_deleted: false,
-        };
-        self.route_db.route_for.insert(*peer_ip, ri);
     }
     pub fn remove_dynamic_peer(&mut self, peer_ip: &Ipv4Addr) {
+        self.peer.remove(peer_ip);
         self.peers.remove(peer_ip);
         self.recalculate_routes();
-
-        if let Some(ref mut ri) = self.route_db.route_for.get_mut(peer_ip) {
-            ri.to_be_deleted = true;
-        } else {
-            panic!("should not happe");
-        }
     }
 
     pub fn get_route_changes(&mut self) -> Vec<RouteChange> {
@@ -162,26 +167,49 @@ impl NetworkManager {
     pub fn db_version(&self) -> usize {
         self.route_db.version
     }
-    pub fn analyze_advertisement(&mut self, udp_packet: &UdpPacket) -> Option<Ipv4Addr> {
-        use UdpPacket::*;
-        match udp_packet {
-            RouteDatabaseRequest { .. } => None,
-            RouteDatabase { .. } => None,
-            Advertisement {
-                wg_ip,
-                routedb_version,
-                ..
-            } => {
-                if let Some(peer_route_db) = self.peer_route_db.get(wg_ip) {
-                    if peer_route_db.version == *routedb_version {
-                        return None;
-                    }
-                    self.peer_route_db.remove(wg_ip);
-                }
-                Some(*wg_ip)
-            }
+    pub fn analyze_advertisement_for_new_peer(
+        &mut self,
+        advertisement: &AdvertisementPacket,
+        admin_port: u16,
+    ) -> Option<Ipv4Addr> {
+        self.fifo_dead.push(advertisement.wg_ip);
+        self.fifo_ping.push(advertisement.wg_ip);
+        let lastseen = crate::util::now();
+        if self
+            .peer
+            .insert(
+                advertisement.wg_ip,
+                DynamicPeer {
+                    wg_ip: advertisement.wg_ip,
+                    local_ip_list: advertisement.local_ip_list.clone(),
+                    local_admin_port: advertisement.local_admin_port,
+                    local_wg_port: advertisement.local_wg_port,
+                    public_key: advertisement.public_key.clone(),
+                    name: advertisement.name.to_string(),
+                    endpoint: advertisement.endpoint,
+                    admin_port,
+                    lastseen,
+                },
+            )
+            .is_none()
+        {
+            Some(advertisement.wg_ip)
+        } else {
+            None
         }
     }
+    pub fn analyze_advertisement(
+        &mut self,
+        advertisement: &AdvertisementPacket,
+    ) -> Option<Ipv4Addr> {
+        if let Some(peer_route_db) = self.peer_route_db.get(&advertisement.wg_ip) {
+            if peer_route_db.version == advertisement.routedb_version {
+                return None;
+            }
+            self.peer_route_db.remove(&advertisement.wg_ip);
+        }
+        Some(advertisement.wg_ip)
+    }
     pub fn provide_route_database(&self) -> Vec<UdpPacket> {
         let mut known_routes = vec![];
         for ri in self.route_db.route_for.values().filter(|ri| ri.issued) {
@@ -195,47 +223,36 @@ impl NetworkManager {
         );
         vec![p]
     }
-    pub fn process_route_database(&mut self, udp_packet: UdpPacket) -> bool {
-        use UdpPacket::*;
+    pub fn process_route_database(&mut self, req: RouteDatabasePacket) -> bool {
         let mut need_routes_update = false;
-        match udp_packet {
-            Advertisement { .. } => {}
-            RouteDatabaseRequest { .. } => {}
-            RouteDatabase {
-                sender,
-                known_routes,
-                routedb_version,
-                nr_entries,
-            } => {
-                debug!("RouteDatabase from {}: {:?}", sender, known_routes);
-                if let Some(mut peer_route_db) = self.peer_route_db.remove(&sender) {
-                    if nr_entries == peer_route_db.nr_entries {
-                        if routedb_version == peer_route_db.version {
-                            for ri in known_routes {
-                                peer_route_db.route_for.insert(ri.to, ri);
-                            }
-                            need_routes_update = nr_entries == peer_route_db.route_for.len();
-                            self.peer_route_db.insert(sender, peer_route_db);
-                        } else {
-                            warn!("Mismatch of route db version");
-                        }
-                    } else {
-                        warn!("Mismatch of nr_entries");
+        debug!("RouteDatabase from {}: {:?}", req.sender, req.known_routes);
+        if let Some(mut peer_route_db) = self.peer_route_db.remove(&req.sender) {
+            if req.nr_entries == peer_route_db.nr_entries {
+                if req.routedb_version == peer_route_db.version {
+                    for ri in req.known_routes {
+                        peer_route_db.route_for.insert(ri.to, ri);
                     }
+                    need_routes_update = req.nr_entries == peer_route_db.route_for.len();
+                    self.peer_route_db.insert(req.sender, peer_route_db);
                 } else {
-                    let routes = known_routes
-                        .iter()
-                        .map(|e| (e.to, e.clone()))
-                        .collect::<HashMap<Ipv4Addr, RouteInfo>>();
-                    let peer_route_db = PeerRouteDB {
-                        version: routedb_version,
-                        nr_entries,
-                        route_for: routes,
-                    };
-                    need_routes_update = nr_entries == peer_route_db.route_for.len();
-                    self.peer_route_db.insert(sender, peer_route_db);
+                    warn!("Mismatch of route db version");
                 }
+            } else {
+                warn!("Mismatch of nr_entries");
             }
+        } else {
+            let routes = req
+                .known_routes
+                .iter()
+                .map(|e| (e.to, e.clone()))
+                .collect::<HashMap<Ipv4Addr, RouteInfo>>();
+            let peer_route_db = PeerRouteDB {
+                version: req.routedb_version,
+                nr_entries: req.nr_entries,
+                route_for: routes,
+            };
+            need_routes_update = req.nr_entries == peer_route_db.route_for.len();
+            self.peer_route_db.insert(req.sender, peer_route_db);
         }
         if need_routes_update {
             self.recalculate_routes();
@@ -249,6 +266,7 @@ impl NetworkManager {
         //    peer route_db, if valid
         let mut new_routes: HashMap<Ipv4Addr, Option<Gateway>> = HashMap::new();
 
+        // Dynamic peers are ALWAYS reachable without a gateway
         for peer in self.peers.iter() {
             new_routes.insert(*peer, None);
         }
@@ -355,4 +373,38 @@ impl NetworkManager {
 
         ips
     }
+    pub fn knows_peer(&mut self, wg_ip: &Ipv4Addr) -> bool {
+        self.peer.contains_key(wg_ip)
+    }
+    pub fn output(&self) {
+        for peer in self.peer.values() {
+            info!("{:?}", peer);
+        }
+    }
+    pub fn check_timeouts(&mut self, limit: u64) -> HashSet<Ipv4Addr> {
+        let mut dead_peers = HashSet::new();
+        while let Some(wg_ip) = self.fifo_dead.first().as_ref() {
+            if let Some(peer) = self.peer.get(*wg_ip) {
+                if crate::util::now() - peer.lastseen < limit {
+                    break;
+                }
+                dead_peers.insert(**wg_ip);
+            }
+            self.fifo_dead.remove(0);
+        }
+        dead_peers
+    }
+    pub fn check_ping_timeouts(&mut self, limit: u64) -> HashSet<(Ipv4Addr, u16)> {
+        let mut ping_peers = HashSet::new();
+        while let Some(wg_ip) = self.fifo_ping.first().as_ref() {
+            if let Some(peer) = self.peer.get(*wg_ip) {
+                if crate::util::now() - peer.lastseen < limit {
+                    break;
+                }
+                ping_peers.insert((**wg_ip, peer.admin_port));
+            }
+            self.fifo_ping.remove(0);
+        }
+        ping_peers
+    }
 }
